package estructuras.arboles;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;

/**
 * Nodo de un rbol binario ordenado
 * @author Alvaro
 *
 * @param <K>
 * @param <V>
 */
public class NodoABO<K extends Comparable<K>, V> implements Serializable, Map.Entry<K, V>
{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Identificador del nodo
	 */
	private K id;
	
	/**
	 * El elemento almacenado en el nodo
	 */
	private V elem;
	
	/**
	 * El hijo izquierdo del nodo
	 */
	private NodoABO<K, V> izq;
	
	/**
	 * El hijo derecho del nodo
	 */
	private NodoABO<K, V> der;
	
	/**
	 * Crea un nodo a partir de su informacin
	 * @param nId el id del nodo. nId != null
	 * @param nElem el elemento almacenado en el nodo. nElem != null
	 */
	public NodoABO(K nId, V nElem)
	{
		id = nId;
		elem = nElem;
		izq = null;
		der = null;
	}
	
	/**
	 * Devuelve el identificador del nodo
	 * @return id
	 */
	public K getKey() {
		return id;
	}

	/**
	 * Devuelve el elemento del nodo
	 * @return elem
	 */
	public V getValue() {
		return elem;
	}

	/**
	 * Cambia el elemento del nodo
	 * @param nElem el nuevo elemento del nodo
	 * @return el elemento previo del nodo
	 */
	public V setValue(V nElem)
	{
		V antiguo = elem;
		elem = nElem;
		return antiguo;
	}

	/**
	 * Indica si el subrbol que inicia en el nodo contiene el id por parmetro
	 * @param buscado el identificador buscado en el subrbol
	 * @return true en caso que el sub rbol tenga el id o false en caso contrario
	 */
	public boolean containsKey(K buscado)
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Devuelve el elemento del nodo con el id dado por parmetro
	 * @param buscado el id del elemento buscado
	 * @return el elemento del nodo con el id dado o null en caso que el id no exista en el sub rbol
	 */
	public V get(K buscado) 
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Agrega un nuevo nodo al sub rbol que inica en el nodo de manera ordenada segn el id
	 * @param nId el identificador del nodo
	 * @param nElem el elemento almacenado en el nodo
	 * @throws IllegalArgumentException se lanza en caso que ya exista un nodo con el identificador dado
	 */
	public void put(K nId, V nElem) throws IllegalArgumentException 
	{
		//TODO complete segn la documentacin
		
		
	}

	/**
	 * Indica el tamao del sub rbol que inica en el nodo
	 * @return cantidad de nodos en el sub rbol
	 */
	public int size()
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Devuelve el id menor del sub rbol que inicia en el nodo
	 * @return el id menor
	 */
	public K firstKey() 
	{
		//TODO complete segn la documentacin
		
	}
	
	/**
	 * Devuelve el id mayor del sub rbol que inicia en el nodo
	 * @return el id mayor
	 */
	public K lastKey() 
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Devuelve el conjunto de ids del sub rbol que empieza en el nodo
	 * @param s el conjunto donde se acumulan los id
	 */
	public void keyset(Set<K> s) 
	{
		if(izq != null)
		{
			izq.keyset(s);
		}
		s.add(id);
		if(der != null)
		{
			der.keyset(s);
		}
		
	}
	
	/**
	 * Construye un rbol binario ordenado con los nodos (copias) con id menor al lmite por parmetro
	 * @param limite el id lmite del rbol
	 * @param map el rbol donde se acumulan los nodos con id menor al lmite
	 */
	public void headMap(K limite, SortedMap<K, V> map) 
	{
		//TODO complete segn la documentacin
		

	}
	
	/**
	 * Construye un rbol binario ordenado con los nodos (copias) con id mayor o igual al lmite por parmetro
	 * @param limite el id lmite del rbol
	 * @param map el rbol donde se acumulan los nodos con id mayor o igual al lmite
	 */
	public void tailMap(K limite, SortedMap<K, V> map) 
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Indica si el sub rbol que inicia en el nodo tiene el elemento por parmetro
	 * @param buscado el elemento buscado
	 * @return true en caso de que el elemento exista en el sub rbol o false en caso contrario
	 */
	public boolean containsValue(V buscado) 
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Construye un conjunto con los nodos del rbol en inorden
	 * @param s el conjunto donde se acumulan los nodos del rbol
	 */
	public void entrySet(Set<java.util.Map.Entry<K, V>> s) 
	{
		if(izq != null)
		{
			izq.entrySet(s);
		}
		s.add(this);
		if(der != null)
		{
			der.entrySet(s);
		}
		
	}

	/**
	 * Construye una coleccin con los elementos del rbol en inorden
	 * @param s la coleccin donde se acumulan los elementos del rbol
	 */
	public void values(Collection<V> s) 
	{
		//TODO complete segn la documentacin
		
		
	}

	/**
	 * Construye un rbol binario ordenado con los nodos (copias) cuya id est entre los lmites dados
	 * @param limiteInf limite inferior del rbol (excluyente)
	 * @param limiteSup lmite superior del rbol (incluyeten)
	 * @param s el rbol donde se acumulan los nodos cuyo id est entre los lmites dados
	 */
	public void subMap(K limiteInf, K limiteSup, ArbolBO<K, V> s) 
	{
		//TODO complete segn la documentacin
		
		
	}
	
	/**
	 * Elimina un nodo del rbol
	 * @param buscado el identificador del nodo a eliminar
	 * @return el nodo que debe reemplzar a cada nodo despus de la eliminacin
	 */
	public NodoABO<K,V> remove(K buscado)
	{
		//TODO complete segn la documentacin
		
		
		
	}

	/**
	 * Inidca si el nodo es una hoja
	 * @return true en caso que el nodo sea una hoja o false en caso contrario
	 */
	public boolean esHoja() 
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Devuelve el nodo de menor id dentro del sub rbol que inicia en el nodo
	 * @return el nodo de menor id
	 */
	public NodoABO<K, V> darMenor() 
	{
		//TODO complete segn la documentacin
		
	}
	

}
