/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: cupiTaxonomia
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */

package uniandes.cupi2.cupiTaxonomia.interfaz;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import uniandes.cupi2.cupiTaxonomia.mundo.Taxon;
import uniandes.cupi2.cupiTaxonomia.mundo.TipoTaxon;

/**
 * Dilogo para agregar un nuevo taxn.
 */
public class DialogoAgregarTaxon extends JDialog implements ActionListener
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Constante que representa el comando para agregar.
     */
    private final static String AGREGAR = "Agregar";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Ventana principal de la aplicacin.
     */
    private InterfazCupiTaxonomia principal;

    /**
     * Tipo del taxn a agregar.
     */
    private TipoTaxon tipoTaxon;

    // -----------------------------------------------------------------
    // Atributos de la Interfaz
    // -----------------------------------------------------------------

    /**
     * ComboBox que guarda los posibles taxones padres.
     */
    private JComboBox comboPadres;

    /**
     * Campo de texto con el nombre del taxn.
     */
    private JTextField txtNombre;

    /**
     * Botn Agregar.
     */
    private JButton btnAgregar;

    // -----------------------------------------------------------------
    // Constructor
    // -----------------------------------------------------------------

    /**
     * Construye un dilogo para agregar un taxn.
     * <b> post: </b> Se crea el dilogo con todos sus elementos grficos.
     * @param pPrincipal Ventana principal de la aplicacin. pPrincipal != null.
     * @param pTipo Tipo del taxn a agregar. pTipo != null && pTipo != "".
     */
    public DialogoAgregarTaxon( InterfazCupiTaxonomia pPrincipal, TipoTaxon pTipo )
    {
        principal = pPrincipal;

        tipoTaxon = pTipo;

        setLayout( new BorderLayout( ) );
        setSize( 250, 120 );
        setModal( true );
        setLocationRelativeTo( principal );
        setTitle( "Agregar " +tipoTaxon);
        setResizable( false );

        JPanel panel1 = new JPanel( );
        panel1.setPreferredSize( new Dimension( 0, 50 ) );
        panel1.setLayout( new GridLayout( 2, 2 ) );

        Collection<Taxon> listaPadres = principal.darTaxonesTipo( tipoTaxon.darNivel() - 1 );

        panel1.add( new JLabel( " Padre: " ) );
        comboPadres = new JComboBox( listaPadres.toArray( ) );
        panel1.add( comboPadres );

        panel1.add( new JLabel( " Nombre: " ) );
        txtNombre = new JTextField( );
        panel1.add( txtNombre );

        JPanel panel2 = new JPanel( );
        panel2.setLayout( new BorderLayout( ) );

        add( panel1, BorderLayout.CENTER );

        btnAgregar = new JButton( AGREGAR );
        btnAgregar.setActionCommand( AGREGAR );
        btnAgregar.addActionListener( this );

        add( btnAgregar, BorderLayout.SOUTH );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Manejo de los eventos de los botones.
     * @param pEvento Accin que gener el evento. pEvento != null.
     */
    public void actionPerformed( ActionEvent pEvento )
    {
        String comando = pEvento.getActionCommand( );
        if( comando.equals( AGREGAR ) )
        {
            Taxon tpadre = ( Taxon )comboPadres.getSelectedItem( );
            String padre = tpadre.darNombre();
            String nombre = txtNombre.getText( );

            if( padre != null && !padre.isEmpty( ) && nombre != null && !nombre.isEmpty( ) )
            {
            	
                principal.agregarTaxon( padre, tipoTaxon, nombre );
                this.dispose( );
            }
            else
            {
                JOptionPane.showMessageDialog( this, "Por favor ingrese la informacin completa del taxn.", "Agregar taxn", JOptionPane.ERROR_MESSAGE );
            }
        }
    }
}