/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: cupiTaxonomia
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */

package uniandes.cupi2.cupiTaxonomia.interfaz;

import java.awt.*;

import javax.swing.*;

import archivos.Serializador;

import java.util.Collection;

import uniandes.cupi2.cupiTaxonomia.mundo.ArbolTaxonomico;
import uniandes.cupi2.cupiTaxonomia.mundo.SerVivo;
import uniandes.cupi2.cupiTaxonomia.mundo.Taxon;
import uniandes.cupi2.cupiTaxonomia.mundo.TipoTaxon;

/**
 * Ventana principal de la aplicacin.
 */
public class InterfazCupiTaxonomia extends JFrame
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Constante que representa la ruta del archivo con la informacin del rbol taxonmico.
     */
    private static final String RUTA_ARCHIVO = "./data/arbolTaxonomico.data";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase principal del mundo.
     */
    private ArbolTaxonomico arbol;

    // -----------------------------------------------------------------
    // Atributos de la interfaz
    // -----------------------------------------------------------------

    /**
     * Panel con la imagen encabezado.
     */
    private PanelImagen panelImagen;

    /**
     * Panel con las opciones de extensin.
     */
    private PanelExtension panelExtension;

    /**
     * Panel con la lista de taxones.
     */
    private PanelTaxones panelTaxones;

    /**
     * Panel con la lista de seres vivos.
     */
    private PanelSeresVivos panelSeresVivos;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Construye la ventana principal. <br>
     * <b>post: </b> Se ha creado la ventana con todos sus pneles.
     */
    public InterfazCupiTaxonomia( )
    {
        // Crea la clase principal
        try
        {
            arbol = new ArbolTaxonomico(  );
            arbol = Serializador.cargar(RUTA_ARCHIVO, arbol);

            // Construye la forma
            setLayout( new BorderLayout( ) );
            setSize( 650, 530 );
            setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
            setTitle( " CupiTaxonoma " );
            setResizable( false );

            // Creacin de los paneles aqu
            panelImagen = new PanelImagen( );
            add( panelImagen, BorderLayout.NORTH );

            JPanel aux = new JPanel( );
            aux.setLayout( new BorderLayout( ) );

            panelTaxones = new PanelTaxones( this );
            aux.add( panelTaxones, BorderLayout.WEST );

            panelSeresVivos = new PanelSeresVivos( this );
            aux.add( panelSeresVivos, BorderLayout.CENTER );

            add( aux, BorderLayout.CENTER );

            panelExtension = new PanelExtension( this );
            add( panelExtension, BorderLayout.SOUTH );

            // Centrar la ventana
            setLocationRelativeTo( null );

            actualizarTaxones( );
        }
        catch( Exception e )
        {
        	e.printStackTrace();
            JOptionPane.showMessageDialog( this, e.getMessage( ), "CupiTaxonomia", JOptionPane.ERROR_MESSAGE );
            System.exit( ERROR );
        }
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Actualiza la lista de taxones mostrados.
     */
    private void actualizarTaxones( )
    {
        panelTaxones.refrescarArbol( arbol.darTaxonRaiz( ) );
        panelExtension.actualizar( arbol.darNumTaxones( ), arbol.darNumSeresVivos( ) );
    }

    /**
     * Muestra la lista de taxones en preorden.
     */
    public void verTaxonesPreorden( )
    {
        Collection<Taxon> taxones = arbol.darTaxonesPreorden( );
        DialogoVerTaxones dialogo = new DialogoVerTaxones( taxones );
        dialogo.setLocationRelativeTo( this );
        dialogo.setVisible( true );
    }

    /**
     * Muestra la lista de taxones en postorden.
     */
    public void verTaxonesPostorden( )
    {
        Collection<Taxon> taxones = arbol.darTaxonesPostorden( );
        DialogoVerTaxones dialogo = new DialogoVerTaxones( taxones );
        dialogo.setLocationRelativeTo( this );
        dialogo.setVisible( true );
    }

    /**
     * Actualiza la lista de seres vivos con los seres del taxn dado
     * @param pTaxon Taxn del rbol taxonmico. pTaxon != null
     */
    public void actualizarSeresVivos( Taxon pTaxon )
    {
        Collection<SerVivo> seres = arbol.darSeresVivos( pTaxon.darNombre( ) );
        panelSeresVivos.refrescar( seres );
    }

    /**
     * Agrega un nuevo taxn al rbol taxonmico.
     * @param pNombrePadre Nombre del taxn padre del taxn a agregar. pNombrePadre != null y pNombrePadre != "".
     * @param pTipo Tipo del taxn a agregar. pTipo pertenece a {Taxon.LUCA, Taxon.DOMINIO, Taxon.REINO, Taxon.FILO, Taxon.CLASE, Taxon.ORDEN, Taxon.FAMILIA, Taxon.GENERO, Taxon.ESPECIE}.
     * @param pNombreTaxon Nombre del taxn a agregar. pNombreTaxon != null y pNombreTaxon != "".
     */
    public void agregarTaxon( String pNombrePadre, TipoTaxon pTipo, String pNombreTaxon )
    {
        try
        {
            arbol.agregarTaxon( pNombrePadre, pTipo, pNombreTaxon );
            actualizarTaxones( );
        }
        catch( Exception e )
        {
            JOptionPane.showMessageDialog( this, e.getMessage( ), "Agregar taxn", JOptionPane.ERROR_MESSAGE );
        }
    }

    /**
     * Elimina el taxn con el nombre y el tipo dado del rbol taxonmico.
     * @param pTipo Tipo del taxn a eliminar. pTipo pertenece a {Taxon.LUCA, Taxon.DOMINIO, Taxon.REINO, Taxon.FILO, Taxon.CLASE, Taxon.ORDEN, Taxon.FAMILIA, Taxon.GENERO, Taxon.ESPECIE}.
     * @param pNombre Nombre del taxn a eliminar. pNombre != null y pNombre != "".
     */
    public void eliminarTaxon(Taxon taxon )
    {
        arbol.eliminarTaxon( taxon );
        actualizarTaxones( );
    }

    /**
     * Agrega un ser vivo al rbol taxonmico. <br>
     * <b> pre: </b> El taxn al que se va a gregar el ser vivo es de tipo Especie.
     * @param pNombreTaxon Nombre del taxn al que se va a agregar el ser vivo. pNombreTaxon != null y pNombreTaxon != "".
     * @param pNombreComun Nombre comn del ser vivo. pNombreComun != null y pNombreComun != "".
     * @param pNombreCientifico Nombre cientfico del ser vivo. pNombreCientifico != null y pNombreCientifico != "".
     * @param pCaracteristicas Caractersticas del ser vivo. pCaracteristicas != null y pCaracteristicas != "".
     * @param pImagen Ruta con la imagen del ser vivo. pImagen != null y pImagen != "".
     */
    public void agregarSerVivo( String pNombreTaxonPadre, String pNombreComun, String pNombreCientifico, String pCaracteristicas, String pImagen )
    {
        try
        {
            arbol.agregarSerVivo( pNombreTaxonPadre, pNombreComun, pNombreCientifico, pCaracteristicas, pImagen );
            actualizarTaxones( );
        }
        catch( Exception e )
        {
            JOptionPane.showMessageDialog( this, e.getMessage( ), "Agregar ser vivo", JOptionPane.ERROR_MESSAGE );
        }
    }

    /**
     * Retorna la lista con los nombres de los taxones del tipo dado.
     * @param pTipo Tipo de los taxones. pTipo pertenece a {Taxon.LUCA, Taxon.DOMINIO, Taxon.REINO, Taxon.FILO, Taxon.CLASE, Taxon.ORDEN, Taxon.FAMILIA, Taxon.GENERO, Taxon.ESPECIE}.
     * @return Lista con los nombre de los taxones del tipo dado.
     */
    public Collection<Taxon> darTaxonesTipo( int pTipo )
    {
        return arbol.darTaxonesTipo( pTipo );
    }

    /**
     * Mtodo que se llama cuando se cierra la ventana principal de la aplicacin.<br>
     * Antes de cerrar se guarda el estado del programa.<br>
     * Si se produce un error se muestra un mensaje que informe al usuario.
     */
    public void dispose( )
    {
        try
        {
            Serializador.guardar(RUTA_ARCHIVO, arbol);
            super.dispose( );
        }
        catch( Exception e )
        {
            setVisible( true );
            int respuesta = JOptionPane.showConfirmDialog( this, e.getMessage( ) + "\n Quiere cerrar el programa sin salvar?", "CupiTaxonomia", JOptionPane.YES_NO_OPTION );
            if( respuesta == JOptionPane.YES_OPTION )
            {
                super.dispose( );
            }
        }
    }

    // -----------------------------------------------------------------
    // Puntos de Extensin
    // -----------------------------------------------------------------

    /**
     * Mtodo para la extensin 1.
     */
    public void reqFuncOpcion1( )
    {
        String resultado = arbol.metodo1( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    /**
     * Mtodo para la extensin 2.
     */
    public void reqFuncOpcion2( )
    {
        String resultado = arbol.metodo2( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    // -----------------------------------------------------------------
    // Main
    // -----------------------------------------------------------------

    /**
     * Este mtodo ejecuta la aplicacin, creando una nueva interfaz.
     * @param pArgs Argumentos de la aplicacin. pArgs != null.
     */
    public static void main( String[] pArgs )
    {
        // Unifica la interfaz para Mac y para Windows.
        try 
        {
            UIManager.setLookAndFeel( UIManager.getCrossPlatformLookAndFeelClassName( ) );
        } 
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        InterfazCupiTaxonomia interfaz = new InterfazCupiTaxonomia( );
        interfaz.setVisible( true );
    }

	public void buscarMenores() 
	{
		String s = JOptionPane.showInputDialog(this, "Ingrese el nombre de un ser vivo.", "Buscar menores", JOptionPane.QUESTION_MESSAGE);
		if(s != null )
		{
			Collection<SerVivo> seres = arbol.darMenores(s);
			String msj = "Seres nombre menor que " + s + ":";
			for (SerVivo serVivo : seres) {
				
				msj += "\n"+serVivo.toString();
			}
			JOptionPane.showMessageDialog(this, msj, "Menores que", JOptionPane.INFORMATION_MESSAGE);
			
			
		}
	}

	public void buscarMayores() {
		String s = JOptionPane.showInputDialog(this, "Ingrese el nombre de un ser vivo.", "Buscar mayores o iguales", JOptionPane.QUESTION_MESSAGE);
		if(s != null)
		{
			
			Collection<SerVivo> seres = arbol.darMayoresIguales(s);
			String msj = "Seres nombre mayor o igual que " + s + ":";
			for (SerVivo serVivo : seres) {
				
				msj += "\n"+serVivo.toString();
			}
			JOptionPane.showMessageDialog(this, msj, "Mayores iguales que", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	public void buscarEntre() 
	{
		String s1 = JOptionPane.showInputDialog(this, "Ingrese el nombre (lmite inferior) de un ser vivo.", "Buscar entre", JOptionPane.QUESTION_MESSAGE);
		String s2 = JOptionPane.showInputDialog(this, "Ingrese el nombre (lmite superior) de un ser vivo.", "Buscar entre", JOptionPane.QUESTION_MESSAGE);
		if(s1 != null && s2 != null)
		{
			
			Collection<SerVivo> seres = arbol.darEntre(s1, s2);
			String msj = "Seres nombre entre " + s1 + " y " + s2 + ":";
			for (SerVivo serVivo : seres) {
				
				msj += "\n"+serVivo.toString();
			}
			JOptionPane.showMessageDialog(this, msj, "Entre", JOptionPane.INFORMATION_MESSAGE);
			
		}
	}
}
