/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: cupiTaxonomia
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiTaxonomia.interfaz;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import uniandes.cupi2.cupiTaxonomia.mundo.TipoTaxon;

/**
 * Panel para el manejo de extensiones.
 */
public class PanelExtension extends JPanel implements ActionListener
{

    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Constante que representa el comando para agregar un taxn.
     */
    private static final String AGREGAR_TAXON = "Agregar taxn";

    /**
     * Constante que representa el comando para agregar un ser vivo.
     */
    private final static String AGREGAR_SER = "Agregar ser vivo";

    /**
     * Constante que representa el comando para la Opcin 1.
     */
    private static final String OPCION_1 = "OPCION_1";

    /**
     * Constante que representa el comando para la Opcin 2.
     */
    private static final String OPCION_2 = "OPCION_2";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Ventana principal de la aplicacin.
     */
    private InterfazCupiTaxonomia principal;

    // -----------------------------------------------------------------
    // Atributos de interfaz
    // -----------------------------------------------------------------

    /**
     * Botn para agregar un taxn.
     */
    private JButton btnAgregarTaxon;

    /**
     * Botn para agregar un ser vivo.
     */
    private JButton btnAgregarServivo;

    /**
     * Campo de texto con el nmero de taxones.
     */
    private JTextField txtTaxones;

    /**
     * Campo de texto con el nmero de seres vivos.
     */
    private JTextField txtSeresVivos;

    /**
     * Botn para la Opcin 1.
     */
    private JButton btnOpcion1;

    /**
     * Botn para la Opcin 2.
     */
    private JButton btnOpcion2;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Constructor del panel de extensin.
     * <b> post: </b> Se crea el panel con todos sus elementos grficos.
     * @param pPrincipal Ventana principal de la aplicacin. pPrincipal != null.
     */
    public PanelExtension( InterfazCupiTaxonomia pPrincipal )
    {
        principal = pPrincipal;

        setBorder( new TitledBorder( "Opciones" ) );
        setLayout( new GridLayout( 2, 4 ) );

        JLabel lbl1 = new JLabel( "Nmero de taxones: " );
        lbl1.setHorizontalAlignment( JLabel.CENTER );
        add( lbl1 );

        txtTaxones = new JTextField( "0" );
        txtTaxones.setEditable( false );
        txtTaxones.setHorizontalAlignment( JTextField.CENTER );
        add( txtTaxones );

        JLabel lbl2 = new JLabel( "Nmero de seres vivos: " );
        lbl2.setHorizontalAlignment( JLabel.CENTER );
        add( lbl2 );

        txtSeresVivos = new JTextField( "0" );
        txtSeresVivos.setEditable( false );
        txtSeresVivos.setHorizontalAlignment( JTextField.CENTER );
        add( txtSeresVivos );

        // Botn agregar taxn
        btnAgregarTaxon = new JButton( AGREGAR_TAXON );
        btnAgregarTaxon.setActionCommand( AGREGAR_TAXON );
        btnAgregarTaxon.addActionListener( this );
        add( btnAgregarTaxon );

        // Botn agregar ser vivo
        btnAgregarServivo = new JButton( AGREGAR_SER );
        btnAgregarServivo.setActionCommand( AGREGAR_SER );
        btnAgregarServivo.addActionListener( this );
        add( btnAgregarServivo );

        // Botn opcin 1
        btnOpcion1 = new JButton( "Opcin 1" );
        btnOpcion1.setActionCommand( OPCION_1 );
        btnOpcion1.addActionListener( this );
        add( btnOpcion1 );

        // Botn opcin 2
        btnOpcion2 = new JButton( "Opcin 2" );
        btnOpcion2.setActionCommand( OPCION_2 );
        btnOpcion2.addActionListener( this );
        add( btnOpcion2 );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Actualiza los campos de texto con la informacin dada del rbol taxonmico.
     * @param pNumTaxones Nmero de taxones del rbol. pNumTaxones >= 0.
     * @param pNumSeres Nmero de seres vivos del rbol. pNumSeres >= 0.
     */
    public void actualizar( int pNumTaxones, int pNumSeres )
    {
        txtTaxones.setText( "" + pNumTaxones );
        txtSeresVivos.setText( "" + pNumSeres );
    }

    /**
     * Manejo de los eventos de los botones.
     * @param pEvento Accin que gener el evento. pEvento != null.
     */
    public void actionPerformed( ActionEvent pEvento )
    {
        String comando = pEvento.getActionCommand( );
        if( OPCION_1.equals( comando ) )
        {
            principal.reqFuncOpcion1( );
        }
        else if( OPCION_2.equals( comando ) )
        {
            principal.reqFuncOpcion2( );
        }
        else if( comando.equals( AGREGAR_TAXON ) )
        {
        	
            TipoTaxon tipo = (TipoTaxon)JOptionPane.showInputDialog(principal, "Tipo de taxn:", "Agregar taxn", JOptionPane.INFORMATION_MESSAGE, null, TipoTaxon.values(), TipoTaxon.DOMINIO);
            if( tipo != null )
            {
            	if(tipo.equals(TipoTaxon.ESPECIE))
            	{
            		JOptionPane.showMessageDialog(principal, "Para crear especies cree seres vivos", "Error", JOptionPane.ERROR_MESSAGE);
            	}
            	else
            	{
            		DialogoAgregarTaxon dialogo = new DialogoAgregarTaxon( principal, tipo );
            		dialogo.setVisible( true );
            		
            	}
            }
        }
        else if( comando.equals( AGREGAR_SER ) )
        {
            DialogoAgregarSerVivo dialogo = new DialogoAgregarSerVivo( principal );
            dialogo.setVisible( true );
        }
    }

}
