/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: cupiTaxonomia
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */

package uniandes.cupi2.cupiTaxonomia.interfaz;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import uniandes.cupi2.cupiTaxonomia.mundo.SerVivo;

/**
 * Panel que contiene la informacin de los seres vivos del rbol taxonmico.
 */
public class PanelSeresVivos extends JPanel implements ActionListener
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Constante que representa el comando de ver en detalle un ser vivo.
     */
    private final static String VER = "Ver detalle";
    
    private final static String BUSCAR_MENORES = "Menores que";
    
    private final static String BUSCAR_MAYORES = "Mayores que";
    
    private final static String BUSCAR_ENTRE = "Seres entre";
    // -----------------------------------------------------------------
    // Atributos de la Interfaz
    // -----------------------------------------------------------------

    /**
     * Lista con los seres vivos.
     */
    private JList listaSeresVivos;

    /**
     * Panel con un scroll que contiene la lista de seres vivos.
     */
    private JScrollPane scroll;

    /**
     * Botn para ver la informacin de un ser vivo.
     */
    private JButton btnVer;
    
    /**
     * Botn para ver los seres vivos mayores que un nombre
     */
    private JButton btnMayores;
    
    /**
     * Botn para ver los seres vivos menores que un nombre
     */
    private JButton btnMenores;
    
    /**
     * Botn para ver los seres vivos entre 2 nombres
     */
    private JButton btnEntre;
    
    /**
     * Botn para ver los seres vivos mayores que un nombre
     */
    private InterfazCupiTaxonomia principal;

    // -----------------------------------------------------------------
    // Constructor
    // -----------------------------------------------------------------

    /**
     * Constructor del panel de visualizacin de los seres vivos.
     * <b> post: </b> Se crea el panel con todos sus elementos grficos.
     */
    public PanelSeresVivos( InterfazCupiTaxonomia interfaz)
    {
        setLayout( new BorderLayout( ) );
        setBorder( new CompoundBorder( new EmptyBorder( 0, 5, 0, 5 ), new TitledBorder( " Seres vivos " ) ) );
        setPreferredSize( new Dimension( 250, 0 ) );
        
        principal = interfaz;

        listaSeresVivos = new JList( );
        listaSeresVivos.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

        DefaultListCellRenderer renderer = new SeresVivosListRenderer( );
        listaSeresVivos.setCellRenderer( renderer );

        scroll = new JScrollPane( listaSeresVivos );
        scroll.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        scroll.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
        scroll.setBorder( new CompoundBorder( new EmptyBorder( 3, 3, 3, 3 ), new LineBorder( Color.BLACK, 1 ) ) );

        
        btnVer = new JButton( VER );
        btnVer.setActionCommand( VER );
        btnVer.addActionListener( this );
        
        btnMayores = new JButton( BUSCAR_MAYORES );
        btnMayores.setActionCommand( BUSCAR_MAYORES );
        btnMayores.addActionListener( this );

        
        btnMenores = new JButton( BUSCAR_MENORES );
        btnMenores.setActionCommand( BUSCAR_MENORES );
        btnMenores.addActionListener( this );

        
        btnEntre = new JButton( BUSCAR_ENTRE );
        btnEntre.setActionCommand( BUSCAR_ENTRE );
        btnEntre.addActionListener( this );


        JPanel panelBotones = new JPanel( new GridLayout(2,2,5,5));
        panelBotones.add(btnVer);
        panelBotones.add(btnMayores);
        panelBotones.add(btnMenores);
        panelBotones.add(btnEntre);
        
        add( scroll, BorderLayout.CENTER );
        add( panelBotones, BorderLayout.SOUTH );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Actualiza la lista de libros con la lista recibida por parmetro.
     * @param pNuevaLista La lista con los libros. pNuevaLista != null.
     */
    public void refrescar( Collection<SerVivo> pNuevaLista )
    {
        listaSeresVivos.setListData( pNuevaLista.toArray( ) );
        if( !pNuevaLista.isEmpty( ) )
        {
            listaSeresVivos.setSelectedIndex( 0 );
        }
    }

    /**
     * Manejo de los eventos de los botones.
     * @param pEvento Accin que gener el evento. pEvento != null.
     */
    public void actionPerformed( ActionEvent pEvento )
    {
        String comando = pEvento.getActionCommand( );
        if( comando.equals( VER ) )
        {
            if( listaSeresVivos.getSelectedValue( ) != null )
            {
                SerVivo ser = ( SerVivo )listaSeresVivos.getSelectedValue( );
                DialogoVerSerVivo dialogo = new DialogoVerSerVivo( ser );
                dialogo.setLocationRelativeTo( this );
                dialogo.setVisible( true );
            }
        }
        if( comando.equals( BUSCAR_MENORES ) )
        {
           principal.buscarMenores();
        }
        if( comando.equals( BUSCAR_MAYORES ) )
        {
        	principal.buscarMayores();
        }
        if( comando.equals( BUSCAR_ENTRE ) )
        {
        	principal.buscarEntre();
        }
    }
}
