/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: cupiTaxonomia
 * Autor: EquipoCupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */

package uniandes.cupi2.cupiTaxonomia.mundo;

import java.io.Serializable;

/**
 * Clase que representa un ser vivo del rbol taxonmico.
 */
public class SerVivo implements Serializable
{
    // -------------------------------------------------------------
    // Atributos
    // -------------------------------------------------------------

    /**
     * Constante de serializacin de la clase.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Nombre comn del ser vivo.
     */
    private String nombreComun;

    /**
     * Nombre cientfico del ser vivo.
     */
    private String nombreCientifico;

    /**
     * Caractersticas del ser vivo.
     */
    private String caracteristicas;

    /**
     * Ruta con la imagen del ser vivo.
     */
    private String rutaImagen;

    // -------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------

    /**
     * Construye un nuevo ser vivo con la informacin dada por parmetro. <br>
     * <b> post: </b> El ser vivo se inicializ con los valores de nombre comn, nombre cientfico, caractersticas e imagen dados por parmetro.
     * @param pNombreComun Nombre comn del ser vivo. pNombreComun != null y pNombreComun != "".
     * @param pNombreCientifico Nombre cientfico del ser vivo. pNombreCientifico != null y pNombreCientifico != "".
     * @param pCaracteristicas Caractersticas del ser vivo. pCaracteristicas != null y pCaracteristicas != "".
     * @param pImagen Ruta con la imagen del ser vivo. pImagen != null y pImagen != "".
     */
    public SerVivo( String pNombreComun, String pNombreCientifico, String pCaracteristicas, String pImagen )
    {
        nombreComun = pNombreComun;
        nombreCientifico = pNombreCientifico;
        caracteristicas = pCaracteristicas;
        rutaImagen = pImagen;
    }

    // -------------------------------------------------------------
    // Mtodos
    // -------------------------------------------------------------

    /**
     * Retorna las caractersticas del ser vivo.
     * @return Caractersticas del ser vivo.
     */
    public String darCaracteristicas( )
    {
        return caracteristicas;
    }
    
    /**
     * Retorna el nombre comn del ser vivo.
     * @return Nombre comn del ser vivo.
     */
    public String darNombreComun( )
    {
        return nombreComun;
    }

    /**
     * Retorna el nombre cientfico del ser vivo.
     * @return Nombre cientfico del ser vivo.
     */
    public String darNombreCientifico( )
    {
        return nombreCientifico;
    }


    /**
     * Retorna la ruta de la imagen del ser vivo.
     * @return Ruta de la imagen del ser vivo.
     */
    public String darRutaImagen( )
    {
        return rutaImagen;
    }

    /**
     * Retorna una cadena con el nombre cientfico y el nombre comn del ser vivo.
     * @return La representacin del ser vivo en String: <nombreCientifico> - <nombreComun>.
     */
    public String toString( )
    {
        return nombreCientifico + " - " + nombreComun;
    }
}