/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: cupiTaxonomia
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */

package uniandes.cupi2.cupiTaxonomia.mundo;

import java.io.Serializable;

/**
 * Clase que representa un taxn del rbol taxonmico. <br>
 * <b> inv: </b> <br>
 * nombre != null && nombre != "". <br>
 * tipo pertenece a {LUCA, DOMINIO, REINO, FILO, CLASE, ORDEN, FAMILIA, GENERO, ESPECIE}. <br>
 * La lista de subtaxones est inicializada. <br>
 * Si el tipo del taxn no es ESPECIE no puede tener un ser vivo. <br>
 * Si el tipo del taxn es ESPECIE no puede tener sub-taxones. <br>
 * El tipo de los sub-taxones corresponde al siguiente tipo en la jerarqua taxonmica. <br>
 */
public class Taxon implements Serializable
{

    // -------------------------------------------------------------
    // Atributos
    // -------------------------------------------------------------

    /**
     * Constante de serializacin de la clase.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Nombre del taxn.
     */
    private String nombre;

    /**
     * Tipo del taxn.
     */
    private TipoTaxon tipo;


    // -------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------

    /**
     * Construye una taxn vaco. <br>
     * <b> post: </b> Se inicializa su tipo en LUCA. <br>
     * Se inicializa su nombre en "ltimo antepasado comn universal". <br>
     * La lista de sub-taxones se inicializa vaca. <br>
     * Se inicializa el ser vivo en null.
     */
    public Taxon( )
    {
        tipo = TipoTaxon.LUCA;
        nombre = "ltimo antepasado comn universal";

        verificarInvariante( );
    }

    /**
     * Inicializa un Taxn con la informacin dada por parmetro. <br>
     * <b> post: </b> El taxn se inicializ con los valores de tipo y nombre dados por parmetro. <br>
     * La lista de sub-taxones se inicializa vaca y el ser vivo en null. <br>
     * @param pTipo Tipo del taxn. pTipo pertenece a {LUCA, DOMINIO, REINO, FILO, CLASE, ORDEN, FAMILIA, GENERO, ESPECIE}.
     * @param pNombre Nombre del taxn. pNombre != null && pNombre != "".
     */
    public Taxon( TipoTaxon pTipo, String pNombre )
    {
        tipo = pTipo;
        nombre = pNombre;

        verificarInvariante( );
    }

    // -------------------------------------------------------------
    // Mtodos
    // -------------------------------------------------------------

    /**
     * Retorna el nombre del taxn.
     * @return Nombre del taxn.
     */
    public String darNombre( )
    {
        return nombre;
    }

    /**
     * Retorna el tipo del taxn.
     * @return Tipo del taxn.
     */
    public TipoTaxon darTipo( )
    {
        return tipo;
    }



    /**
     * Retorna una cadena con el tipo y el nombre del taxn.
     * @return La representacin del taxn en String: <tipo del taxn>: <nombre del taxn>.
     */
    public String toString( )
    {
        String toString = tipo + ": " + nombre;
        return toString;
    }

    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

    /**
     * Verifica la invariante de la clase <br>
     * <b> inv: </b> <br>
     * nombre != null && nombre != "". <br>
     * tipo pertenece a {LUCA, DOMINIO, REINO, FILO, CLASE, ORDEN, FAMILIA, GENERO, ESPECIE}. <br>
     * La lista de subtaxones est inicializada. <br>
     * Si el tipo del taxn no es ESPECIE no puede tener un ser vivo. <br>
     * Si el tipo del taxn es ESPECIE no puede tener sub-taxones. <br>
     * El tipo de los sub-taxones corresponde al siguiente tipo en la jerarqua taxonmica. <br>
     */
    private void verificarInvariante( )
    {
        assert nombre != null && !nombre.isEmpty( ) : "El nombre del taxn debe ser inicializado.";
    
      
    }
}